@@ -1,7 +1,7 @@
 /*
  * Main program and I/O for external vile syntax/highlighter programs
  *
- * $Header: /usr/build/vile/vile/RCS/builtflt.c,v 1.60 2007/09/19 21:45:54 tom Exp $
+ * $Header: /usr/build/vile/vile/RCS/builtflt.c,v 1.63 2008/01/22 00:13:57 tom Exp $
  *
  */
 
@@ -356,7 +356,7 @@ flt_error(const char *fmt,...)
 int
 flt_input(char *buffer, int max_size)
 {
-    char *separator = "\n";
+    const char *separator = "\n";
     int need = strlen(separator);
     int used = 0;
 
@@ -432,7 +432,7 @@ flt_puts(const char *string, int length, const char *marker)
 	save_mark(FALSE);
 	if (apply_attribute()) {
 	    REGIONSHAPE save_shape = regionshape;
-	    regionshape = EXACT;
+	    regionshape = rgn_EXACT;
 	    (void) attributeregion();
 	    videoattribute = 0;
 	    regionshape = save_shape;
@@ -507,6 +507,9 @@ flt_start(char *name)
 	set_symbol_table(current_filter->filter_name);
 	current_filter->InitFilter(0);
 	current_filter->DoFilter(stdin);
+#if NO_LEAKS
+	current_filter->FreeFilter();
+#endif
 
 	DOT = save_dot;
 	MK = save_mk;
