@@ -2,7 +2,7 @@
  *	eval.c -- function and variable evaluation
  *	original by Daniel Lawrence
  *
- * $Header: /usr/build/vile/vile/RCS/eval.c,v 1.370 2007/11/22 16:06:23 tom Exp $
+ * $Header: /usr/build/vile/vile/RCS/eval.c,v 1.379 2008/06/01 17:41:02 tom Exp $
  *
  */
 
@@ -48,7 +48,7 @@ static size_t s2size(const char *s);
 static char **init_vars_cmpl(void);
 static char *get_statevar_val(int vnum);
 static const char *s2offset(const char *s, const char *n);
-static int SetVarValue(VWRAP * var, const char *name, const char *value);
+static int SetVarValue(VWRAP * var, char *name, const char *value);
 static void FindVar(char *var, VWRAP * vd);
 static void free_vars_cmpl(void);
 #endif
@@ -65,9 +65,18 @@ makectypelist(int dum1 GCC_UNUSED, void *ptr GCC_UNUSED)
     CHARTYPE k;
     const char *s;
 
+#if OPT_MULTIBYTE
+    /*
+     * Keep the buffer encoding 8-bit to make it work with the character
+     * classes for the narrow locale.
+     */
+    make_local_b_val(curbp, VAL_FILE_ENCODING);
+    set_b_val(curbp, VAL_FILE_ENCODING, enc_8BIT);
+#endif
+
     bprintf("--- Printable Characters for locale %s (%s) ",
-	    NONNULL(vl_locale),
-	    NONNULL(vl_encoding));
+	    NONNULL(vl_real_enc.locale),
+	    NONNULL(vl_real_enc.encoding));
     bpadc('-', term.cols - DOT.o);
     bputc('\n');
 
@@ -180,7 +189,7 @@ match_charclass_regexp(int ch, REGEXVAL * exp)
 }
 
 static int
-update_charclasses(char *tag, int count)
+update_charclasses(const char *tag, int count)
 {
     if (count) {
 	mlwrite("[%s %d character%s]", tag, count, PLURAL(count));
@@ -962,7 +971,7 @@ run_func(int fnum)
 	    arg[i] = error_val;
 	    is_error = TRUE;
 	}
-	tb_free(&result);	/* in case mac_tokval() called us */
+	tb_init(&result, EOS);	/* in case mac_tokval() called us */
 	TPRINTF(("...arg[%d] = '%s'\n", i, arg[i]));
 	if (args_numeric)
 	    nums[i] = scan_long(arg[i]);
@@ -1412,6 +1421,28 @@ FindModeVar(char *var, VWRAP * vd)
 #endif
 }
 
+static char *
+VarName(char *var)
+{
+    char *result;
+
+    if (var && var[0] && var[1]) {
+	switch (*var) {
+	case '$':
+	case '%':
+	case '&':
+	    result = var + 1;
+	    break;
+	default:
+	    result = var;
+	    break;
+	}
+    } else {
+	result = var;
+    }
+    return result;
+}
+
 /*
  * Find a variable's type and name
  */
@@ -1431,18 +1462,18 @@ FindVar(char *var, VWRAP * vd)
 
     switch (var[0]) {
     case '$':			/* check for legal state var */
-	FindModeVar(var + 1, vd);
+	FindModeVar(VarName(var), vd);
 	break;
 
     case '%':			/* check for legal temp variable */
-	vd->v_ptr = lookup_tempvar(var + 1);
+	vd->v_ptr = lookup_tempvar(VarName(var));
 	if (vd->v_ptr) {	/* existing */
 	    vd->v_type = VW_TEMPVAR;
 	} else {		/* new */
 	    beginDisplay();
 	    p = typealloc(UVAR);
 	    if (p &&
-		(p->u_name = strmalloc(var + 1)) != 0) {
+		(p->u_name = strmalloc(VarName(var))) != 0) {
 		p->next = temp_vars;
 		p->u_value = 0;
 		temp_vars = vd->v_ptr = p;
@@ -1533,9 +1564,13 @@ PromptAndSet(const char *name, int f, int n)
 	status = FALSE;
     } else if (vd.v_type == VW_MODE) {
 	VALARGS args;
-	(void) find_mode(curbp, var + 1, -TRUE, &args);
-	set_end_string('=');
-	status = adjvalueset(var + 1, FALSE, TRUE, -TRUE, &args);
+	if (find_mode(curbp, VarName(var), -TRUE, &args)) {
+	    set_end_string('=');
+	    status = adjvalueset(VarName(var), FALSE, TRUE, -TRUE, &args);
+	} else {
+	    mlforce("[Can't find mode '%s']", VarName(var));
+	    status = FALSE;
+	}
     } else {
 	static TBUFF *tmp;
 
@@ -1693,7 +1728,7 @@ set_statevar_val(int vnum, const char *value)
 
 /* figure out what type of variable we're setting, and do so */
 static int
-SetVarValue(VWRAP * var, const char *name, const char *value)
+SetVarValue(VWRAP * var, char *name, const char *value)
 {
     int status;
     int savecmd;
@@ -1713,9 +1748,12 @@ SetVarValue(VWRAP * var, const char *name, const char *value)
 	savecmd = clexec;
 	execstr = TYPECAST(char, value);
 	clexec = TRUE;
-	(void) find_mode(curbp, name + 1, -TRUE, &args);
-	set_end_string('=');
-	status = adjvalueset(name + 1, FALSE, TRUE, -TRUE, &args);
+	if (find_mode(curbp, VarName(name), -TRUE, &args)) {
+	    set_end_string('=');
+	    status = adjvalueset(VarName(name), FALSE, TRUE, -TRUE, &args);
+	} else {
+	    status = FALSE;
+	}
 	execstr = savestr;
 	clexec = savecmd;
 	break;
@@ -1932,7 +1970,7 @@ makecolorlist(int dum1 GCC_UNUSED, void *ptr GCC_UNUSED)
     DOT.o = 0;
 
     if (getregion(&region) == TRUE)
-	attribute_cntl_a_seqs_in_region(&region, FULLLINE);
+	attribute_cntl_a_seqs_in_region(&region, rgn_FULLLINE);
 }
 
 /* ARGSUSED */
@@ -2012,7 +2050,7 @@ render_long(TBUFF **rp, long i)
 char *
 render_boolean(TBUFF **rp, int val)
 {
-    static char *bools[] =
+    static const char *bools[] =
     {"FALSE", "TRUE"};
     return tb_values(tb_scopy(rp, bools[val ? 1 : 0]));
 }
@@ -2235,7 +2273,7 @@ static int
 read_argument(TBUFF **paramp, const PARAM_INFO * info)
 {
     int status = TRUE;
-    char *prompt;
+    const char *prompt;
     TBUFF *temp;
     int (*complete) (DONE_ARGS) = no_completion;
     int save_clexec;
@@ -2501,7 +2539,7 @@ restore_arguments(BUFFER *bp GCC_UNUSED)
  * arguments for '#', and the whole argument list for '*'.  As a special case,
  * return the list with individual words quoted for '@'.
  */
-static char *
+static const char *
 get_argument(const char *name)
 {
     static TBUFF *value;
@@ -2558,14 +2596,14 @@ get_argument(const char *name)
 
 /* the argument simply represents itself */
 static const char *
-simple_arg_eval(char *argp)
+simple_arg_eval(const char *argp)
 {
     return argp;
 }
 
 /* the returned value is the user's response to the specified prompt */
 static const char *
-query_arg_eval(char *argp)
+query_arg_eval(const char *argp)
 {
     argp = user_reply(tokval(argp + 1), error_val);
     return argp ? argp : error_val;
@@ -2573,7 +2611,7 @@ query_arg_eval(char *argp)
 
 /* the returned value is the next line from the named buffer */
 static const char *
-buffer_arg_eval(char *argp)
+buffer_arg_eval(const char *argp)
 {
     argp = next_buffer_line(tokval(argp + 1));
     return argp ? argp : error_val;
@@ -2581,11 +2619,11 @@ buffer_arg_eval(char *argp)
 
 /* expand it as a temp variable */
 static const char *
-tempvar_arg_eval(char *argp)
+tempvar_arg_eval(const char *argp)
 {
     static TBUFF *tmp;
     UVAR *p;
-    char *result;
+    const char *result;
     if ((p = lookup_tempvar(argp + 1)) != 0) {
 	tb_scopy(&tmp, p->u_value);
 	result = tb_values(tmp);
@@ -2599,7 +2637,7 @@ tempvar_arg_eval(char *argp)
  * not an statevar, perhaps it's a mode?
  */
 static const char *
-statevar_arg_eval(char *argp)
+statevar_arg_eval(const char *argp)
 {
     int vnum;
     const char *result;
@@ -2621,7 +2659,7 @@ statevar_arg_eval(char *argp)
 
 /* run a function to evalute it */
 static const char *
-function_arg_eval(char *argp)
+function_arg_eval(const char *argp)
 {
     int fnum = vl_lookup_func(argp);
     return (fnum != ILLEGAL_NUM) ? run_func(fnum) : error_val;
@@ -2629,7 +2667,7 @@ function_arg_eval(char *argp)
 
 /* goto targets evaluate to their line number in the buffer */
 static const char *
-label_arg_eval(char *argp)
+label_arg_eval(const char *argp)
 {
     static TBUFF *label;
     LINE *lp = label2lp(curbp, argp);
@@ -2642,16 +2680,16 @@ label_arg_eval(char *argp)
 
 /* the value of a quoted string begins just past the leading quote */
 static const char *
-qstring_arg_eval(char *argp)
+qstring_arg_eval(const char *argp)
 {
     return argp + 1;
 }
 
 static const char *
-directive_arg_eval(char *argp)
+directive_arg_eval(const char *argp)
 {
     static TBUFF *tkbuf;
-    char *result;
+    const char *result;
 #if !SYS_UNIX
     /*
      * major hack alert -- the directives start with '~'.  on unix, we'd also
@@ -2673,7 +2711,7 @@ directive_arg_eval(char *argp)
     return result;
 }
 
-typedef const char *(ArgEvalFunc) (char *argp);
+typedef const char *(ArgEvalFunc) (const char *argp);
 
 static ArgEvalFunc *eval_func[] =
 {
@@ -2693,7 +2731,7 @@ static ArgEvalFunc *eval_func[] =
 
 /* evaluate to the string-value of a token */
 const char *
-tokval(char *tokn)
+tokval(const char *tokn)
 {
     const char *result;
 
@@ -2771,8 +2809,11 @@ can_evaluate(void)
 int
 evaluate(int f, int n)
 {
+    static char Expression[] = "Expression";
     static PARAM_INFO info =
-    {PT_STR, "Expression", 0};
+    {
+	PT_STR, Expression, 0
+    };
     TBUFF *params = 0, *tok = 0, *cmd = 0;
     char *old, *tmp;
     int code = FALSE;
