@@ -5,7 +5,7 @@
  * reading and writing of the disk are
  * in "fileio.c".
  *
- * $Header: /usr/build/vile/vile/RCS/file.c,v 1.413 2007/11/23 17:21:28 tom Exp $
+ * $Header: /usr/build/vile/vile/RCS/file.c,v 1.415 2008/04/15 00:10:33 tom Exp $
  */
 
 #include "estruct.h"
@@ -1856,7 +1856,7 @@ slowreadf(BUFFER *bp, int *nlinep)
      * Look for UTF-8 encoding when we have the entire buffer, since only a
      * small part of it may be distinct from ASCII.
      */
-    if (b_is_enc_DEFAULT(bp)) {
+    if (b_is_enc_AUTO(bp)) {
 	LINE *lp;
 	int check, found = SORTOFTRUE;
 
@@ -2659,7 +2659,7 @@ create_save_dir(char *dirnam)
 {
     int result = FALSE;
 #if defined(HAVE_MKDIR) && !SYS_MSDOS && !SYS_OS2
-    static char *tbl[] =
+    static const char *tbl[] =
     {
 	0,			/* reserved for $TMPDIR */
 #if SYS_UNIX
