@@ -1,7 +1,7 @@
 /*
  * Common utility functions for vile syntax/highlighter programs
  *
- * $Header: /usr/build/vile/vile/filters/RCS/filters.c,v 1.108 2007/06/02 15:40:09 tom Exp $
+ * $Header: /usr/build/vile/vile/filters/RCS/filters.c,v 1.115 2008/05/25 23:06:02 tom Exp $
  *
  */
 
@@ -40,7 +40,7 @@ struct _keyword {
     char *kw_attr;
     unsigned kw_size;		/* strlen(kw_name) */
     unsigned short kw_flag;	/* nonzero for classes */
-    unsigned short kw_used;	/* nonzero for classes */
+    short kw_used;		/* nonzero for classes */
 };
 
 typedef struct _classes CLASS;
@@ -177,12 +177,14 @@ static void
 ExecAbbrev(char *param)
 {
     zero_or_more = *param;
+    VERBOSE(1, ("set abbrev '%c'\n", zero_or_more));
 }
 
 static void
 ExecBrief(char *param)
 {
     zero_or_all = *param;
+    VERBOSE(1, ("set brief '%c'\n", zero_or_all));
 }
 
 static void
@@ -529,6 +531,13 @@ flt_bfr_length(void)
     return flt_bfr_used;
 }
 
+void
+flt_free(char **p, unsigned *len)
+{
+    FreeAndNull(*p);
+    *len = 0;
+}
+
 void
 flt_free_keywords(char *classname)
 {
@@ -727,7 +736,7 @@ alloc_keyword(const char *ident, const char *attribute, int classflag)
 		&& nxt->kw_attr != 0) {
 #if USE_TSEARCH
 		void **pp;
-		pp = tsearch(nxt, &(current_class->data), compare_data);
+		pp = (void **) tsearch(nxt, &(current_class->data), compare_data);
 		if (pp != 0) {
 		    nxt = *(KEYWORD **) pp;
 		} else {
@@ -762,7 +771,8 @@ insert_keyword(const char *ident, const char *attribute, int classflag)
 		attribute,
 		classflag));
 
-    if ((mark = strchr(ident, zero_or_more)) != 0
+    if (zero_or_more
+	&& (mark = strchr(ident, zero_or_more)) != 0
 	&& (mark != ident)) {
 	if ((temp = strmalloc(ident)) != 0) {
 
@@ -778,7 +788,8 @@ insert_keyword(const char *ident, const char *attribute, int classflag)
 	} else {
 	    CannotAllocate("insert_keyword");
 	}
-    } else if ((mark = strchr(ident, zero_or_all)) != 0
+    } else if (zero_or_all
+	       && (mark = strchr(ident, zero_or_all)) != 0
 	       && (mark != ident)) {
 	if ((temp = strmalloc(ident)) != 0) {
 
@@ -849,6 +860,12 @@ lowercase_of(char *text)
     unsigned n;
     char *result;
 
+#if NO_LEAKS
+    if (text == 0) {
+	flt_free(&name, &used);
+	result = 0;
+    } else
+#endif
     if ((name = do_alloc(name, strlen(text), &used)) != 0) {
 	for (n = 0; text[n] != 0; n++) {
 	    if (isalpha(CharOf(text[n])) && isupper(CharOf(text[n])))
@@ -995,9 +1012,10 @@ void
 filters_leaks(void)
 {
     flt_free_symtab();
-    FreeAndNull(str_keyword_name);
-    FreeAndNull(str_keyword_file);
-    len_keyword_name = 0;
-    len_keyword_file = 0;
+    flt_free(&str_keyword_name, &len_keyword_name);
+    flt_free(&str_keyword_file, &len_keyword_file);
+
+    flt_free(&flt_bfr_text, &flt_bfr_size);
+    (void) lowercase_of(0);
 }
 #endif
