@@ -1,5 +1,5 @@
 /*
- * $Header: /usr/build/vile/vile/filters/RCS/rubyfilt.c,v 1.41 2007/05/26 14:53:12 tom Exp $
+ * $Header: /usr/build/vile/vile/filters/RCS/rubyfilt.c,v 1.45 2008/05/26 00:53:35 tom Exp $
  *
  * Filter to add vile "attribution" sequences to ruby scripts.  This is a
  * translation into C of an earlier version written for LEX/FLEX.
@@ -62,6 +62,7 @@ static char *Ident2_attr;
 static char *Keyword_attr;
 static char *String_attr;
 static char *Number_attr;
+static char *Type_attr;
 
 static int var_embedded(char *);
 
@@ -266,7 +267,29 @@ is_KEYWORD(char *s)
 }
 
 /*
- * pattern: [$]([-_.\/,"\\#%=~|\$?&`'+*\[\];!@<>():]|{CONST}|{VAR})
+ * Context-specific keyword: Match a method-style keyword,
+ * ie. a KEYWORD with a possible suffix of !, ?, or =.
+ * pattern: {KEYWORD}[!?=]?
+ * returns: length of the token
+ */
+static int
+is_MKEYWORD(char *s, int use_eql)
+{
+    int found = 0;
+
+    if ((found = is_KEYWORD(s)) != 0) {
+	if (ATLEAST(s, found)
+	    && (s[found] == '!'
+		|| s[found] == '?'	/* FIXME - should look for '.' */
+		|| (use_eql == 0 && s[found] == '='))) {
+	    ++found;
+	}
+    }
+    return found;
+}
+
+/*
+ * pattern: [$]([-_.\/,"\\=~\$?&`'+*;!@<>:]|{CONST}|{VAR})
  * returns: length of the token, or zero.
  */
 static int
@@ -275,7 +298,7 @@ is_GLOBAL(char *s)
     int found = 0;
 
     if (*s == '$' && MORE(++s)) {
-	if (*s != '\0' && strchr("-_./,\"\\#%=~|$?&`'+*[];!@<>():", *s)) {
+	if (*s != '\0' && strchr("-_./,\"\\=~$?&`'+*;!@<>:", *s)) {
 	    found = 1;
 	} else if (isdigit(CharOf(*s))) {
 	    while (ATLEAST(s, found)
@@ -291,7 +314,7 @@ is_GLOBAL(char *s)
 }
 
 /*
- * pattern: [@]+{KEYWORD}
+ * pattern: [@][@]?{KEYWORD}
  * returns: length of the token, or zero.
  */
 static int
@@ -300,10 +323,9 @@ is_INSTANCE(char *s)
     char *base = s;
     int found = 0;
 
-    if (*s == '@') {
-	while ((*s++ == '@') && MORE(s)) {
-	    ;
-	}
+    if (*s == '@' && MORE(++s)) {
+	if (*s == '@')
+	    s++;
 	if ((found = is_KEYWORD(s)) != 0)
 	    found += (s - base);
     }
@@ -409,6 +431,9 @@ is_NUMBER(char *s, int *err)
 	    if (ch == '0') {
 		if (!ATLEAST(s, 1)) {
 		    radix = 10;
+		} else if (s[1] == 'b') {
+		    radix = 2;
+		    s++;
 		} else if (s[1] == 'x') {
 		    radix = 16;
 		    s++;
@@ -754,6 +779,37 @@ is_Regexp(char *s, int *delim)
     return found;
 }
 
+/*
+ * Parse both the unquoted and quoted symbol types.
+ * This gets treated as a string literal anyway, so
+ * we only call it from is_String.
+ */
+static int
+is_Symbol(char *s, int *delim, int *err)
+{
+    int found = 0;
+
+    if (*s++ == ':') {
+	switch (*s) {
+	case SQUOTE:
+	    if ((found = is_STRINGS(s, err, *s, *s, 1)) != 0)
+		*delim = SQUOTE;
+	    break;
+	case DQUOTE:
+	    if ((found = is_STRINGS(s, err, *s, *s, 0)) != 0)
+		*delim = DQUOTE;
+	    break;
+	default:
+	    found = is_MKEYWORD(s, 0);
+	}
+    }
+
+    if (found != 0)
+	found += 1;
+
+    return found;
+}
+
 /*
  * Parse the various types of quoted strings.
  */
@@ -763,7 +819,7 @@ is_String(char *s, int *delim, int *err)
     int found = 0;
 
     *delim = 0;
-    if (ATLEAST(s, 2)) {
+    if (ATLEAST(s, 2) && (found = is_Symbol(s, delim, err)) == 0) {
 	switch (*s) {
 	case '%':
 	    if (ATLEAST(s, 4)) {
@@ -927,6 +983,8 @@ put_KEYWORD(char *s, int ok, int *had_op)
     s[ok] = '\0';
     attr = keyword_attr(s);
     s[ok] = save;
+    if ((attr == 0 || *attr == '\0') && isupper(CharOf(s[0])))
+	attr = Type_attr;
     flt_puts(s, ok, attr);
     *had_op = (attr == Keyword_attr);
     return s + ok;
@@ -1040,6 +1098,7 @@ do_filter(FILE *input GCC_UNUSED)
     Keyword_attr = class_attr(NAME_KEYWORD);
     Number_attr = class_attr(NAME_NUMBER);
     String_attr = class_attr(NAME_LITERAL);
+    Type_attr = class_attr(NAME_TYPES);
 
     /*
      * Read the whole file into a single string, in-memory.  Rather than
@@ -1089,7 +1148,7 @@ do_filter(FILE *input GCC_UNUSED)
 		    Parsed(this_tok, tBLANK);
 		    flt_puts(s, ok, "");
 		    s += ok;
-		} else if ((ok = is_KEYWORD(s)) != 0) {
+		} else if ((ok = is_MKEYWORD(s, 0)) != 0) {
 		    Parsed(this_tok, tKEYWORD);
 		    s = put_KEYWORD(s, ok, &had_op);
 		    state = eCODE;
@@ -1176,7 +1235,7 @@ do_filter(FILE *input GCC_UNUSED)
 			flt_puts(s, ok, Number_attr);
 		    }
 		    s += ok;
-		} else if ((ok = is_KEYWORD(s)) != 0) {
+		} else if ((ok = is_MKEYWORD(s, 1)) != 0) {
 		    Parsed(this_tok, tKEYWORD);
 		    if (ok == 5 && !strncmp(s, "alias", ok))
 			state = eALIAS;
@@ -1267,3 +1326,10 @@ do_filter(FILE *input GCC_UNUSED)
 	free(the_file);
     }
 }
+
+#if NO_LEAKS
+static void
+free_filter(void)
+{
+}
+#endif
