@@ -3,9 +3,9 @@
  * written for vile by paul fox.
  * rewritten to use regular expressions by T.Dickey
  *
- * Copyright (c) 1990-2007 by Paul Fox and Thomas Dickey
+ * Copyright (c) 1990-2008 by Paul Fox and Thomas Dickey
  *
- * $Header: /usr/build/vile/vile/RCS/finderr.c,v 1.134 2007/11/25 22:03:29 tom Exp $
+ * $Header: /usr/build/vile/vile/RCS/finderr.c,v 1.136 2008/01/25 00:35:12 tom Exp $
  *
  */
 
@@ -178,7 +178,7 @@ get_febuff(void)
 static const char *
 get_token_name(ErrTokens n)
 {
-    char *result;
+    const char *result;
 
     switch (n) {
     case W_VERB:
