@@ -4,7 +4,7 @@
  * Most code probably by Dan Lawrence or Dave Conroy for MicroEMACS
  * Extensions for vile by Paul Fox
  *
- * $Header: /usr/build/vile/vile/RCS/insert.c,v 1.159 2007/09/16 20:43:15 tom Exp $
+ * $Header: /usr/build/vile/vile/RCS/insert.c,v 1.166 2008/05/26 15:17:59 tom Exp $
  */
 
 #include	"estruct.h"
@@ -46,10 +46,18 @@ static int skipindent;
 static int
 past_wrapmargin(int c)
 {
-    int n;
+    int n = b_val(curbp, VAL_WRAPMARGIN);
 
-    if ((n = b_val(curbp, VAL_WRAPMARGIN)) > 0
-	&& (n = (term.cols - (nu_width(curwp) + n))) >= 0) {
+    if (n > 0) {		/* positive, count from the right */
+	n = (term.cols - (nu_width(curwp) + n));
+    } else if (n == 0) {	/* zero, inactive */
+	n = -1;
+    } else {			/* negative, count from the left */
+	n = -n;
+    }
+
+    /* now 'n' is the effective wrapmargin's column */
+    if (n >= 0) {
 	int list = w_val(curwp, WMDLIST);
 
 	/* compute the effective screen column after adding the
@@ -429,7 +437,7 @@ replacechar(int f, int n)
 		if (isbackspace(c)) {	/* vi beeps here */
 		    s = TRUE;	/* replaced with nothing */
 		} else {
-		    t = s = lins_bytes(n, c);
+		    t = s = lins_chars(n, c);
 		}
 	    }
 	}
@@ -884,7 +892,7 @@ inschar(int c, int *backsp_limit_p)
     }
 
     autoindented = -1;
-    return lins_bytes(1, c);
+    return lins_chars(1, c);
 }
 
 #if ! SMALLER
@@ -1355,7 +1363,7 @@ quote_next(int f, int n)
 }
 
 #if OPT_EVAL
-char *
+const char *
 current_modename(void)
 {
     switch (savedmode) {
